/// <reference types="node" />
import { BN, web3, Program } from '@coral-xyz/anchor';
import { MangoMintsRedemption } from '../mango_mints_redemption';
import { Metadata } from './metadata';
type PublicKey = web3.PublicKey;
export type Claim = {
    mint: PublicKey;
    mintProperties: any;
    quantity: bigint;
};
export declare class Distribution {
    private programId;
    private program;
    private merkleTree;
    publicKey: PublicKey;
    number: number;
    merkleRoot: number[];
    authority: PublicKey;
    start: Date;
    duration: number;
    metadataUrl: string;
    metadata: Metadata;
    constructor(programId: PublicKey, program: Program<MangoMintsRedemption>, publicKey: PublicKey, number: number, merkleRoot: number[], authority: PublicKey, start: BN, duration: BN, metadata: number[]);
    loadMetadata(url?: string): Promise<Metadata>;
    getClaims(claimant: PublicKey): Claim[];
    getAllClaims(): Claim[];
    static findAddress(number: number, programId: PublicKey): PublicKey;
    findClaimAccountAddress(claimant: PublicKey): PublicKey;
    findVaultAddress(mint: PublicKey, distribution?: PublicKey): PublicKey;
    getClaimed(claimant?: PublicKey): Promise<PublicKey[] | undefined>;
    getClaimProof(claimant: PublicKey, mint: PublicKey, quantity: bigint): Buffer[];
    makeClaimInstructions(claimant: PublicKey, mint: PublicKey): Promise<web3.TransactionInstruction[]>;
    makeClaimTransaction(claimant: PublicKey, mint: PublicKey): Promise<web3.Transaction>;
    claim(mint: PublicKey): Promise<string>;
    transferToVault(mint: PublicKey, amount: bigint): Promise<string>;
    closeVault(mint: PublicKey, destination?: PublicKey): Promise<string>;
    close(): Promise<string[]>;
    closeClaimAccount(): Promise<void>;
}
export {};
