"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Distribution = void 0;
const anchor_1 = require("@coral-xyz/anchor");
const axios_1 = __importDefault(require("axios"));
const keccak256_1 = __importDefault(require("keccak256"));
const Token = __importStar(require("@solana/spl-token"));
const metadata_1 = require("./metadata");
class Distribution {
    programId;
    program;
    merkleTree;
    publicKey;
    number;
    merkleRoot;
    authority;
    start;
    duration;
    metadataUrl;
    metadata;
    constructor(programId, program, publicKey, number, merkleRoot, authority, start, duration, metadata) {
        this.programId = programId;
        this.program = program;
        this.publicKey = publicKey;
        this.number = number;
        this.merkleRoot = merkleRoot;
        this.authority = authority;
        this.start = new Date(start.toNumber() * 1000);
        this.duration = duration.toNumber();
        this.metadataUrl = Buffer.from(metadata).toString();
        this.program.provider.opts.skipPreflight = true;
    }
    // Load distribution metadata from an external URL and build Merkle tree
    async loadMetadata(url) {
        const res = await (0, axios_1.default)(url ?? this.metadataUrl);
        // todo: validate
        this.metadata = new metadata_1.Metadata(res.data);
        this.merkleTree = this.metadata.toMerkleTree();
        const computedRoot = this.merkleTree.getRoot();
        const root = Buffer.from(this.merkleRoot);
        if (!root.equals(computedRoot)) {
            throw new Error(`Merkle Root Mismatch: ${computedRoot.toString('hex')} != ${root.toString('hex')}`);
        }
        return this.metadata;
    }
    getClaims(claimant) {
        const claims = this.metadata.claims.filter((c) => c.claimant == claimant.toBase58());
        return claims.map((c) => {
            const mint = this.metadata.mints[c.mintIndex];
            return {
                mint: new anchor_1.web3.PublicKey(mint.address),
                mintProperties: mint.properties,
                quantity: BigInt(c.quantity),
            };
        });
    }
    getAllClaims() {
        return this.metadata.claims.map((c) => {
            const mint = this.metadata.mints[c.mintIndex];
            return {
                mint: new anchor_1.web3.PublicKey(mint.address),
                mintProperties: mint.properties,
                quantity: BigInt(c.quantity),
            };
        });
    }
    static findAddress(number, programId) {
        const distributionNum = Buffer.alloc(4);
        distributionNum.writeUint32LE(number);
        return anchor_1.web3.PublicKey.findProgramAddressSync([Buffer.from('Distribution'), distributionNum], programId)[0];
    }
    findClaimAccountAddress(claimant) {
        return anchor_1.web3.PublicKey.findProgramAddressSync([
            Buffer.from('ClaimAccount'),
            this.publicKey.toBuffer(),
            claimant.toBuffer(),
        ], this.programId)[0];
    }
    findVaultAddress(mint, distribution) {
        return anchor_1.web3.PublicKey.findProgramAddressSync([Buffer.from('Vault'), this.publicKey.toBuffer(), mint.toBuffer()], this.programId)[0];
    }
    async getClaimed(claimant = this.program.provider.publicKey) {
        const publicKey = this.findClaimAccountAddress(claimant);
        const claimAccount = await this.program.account.claimAccount.fetchNullable(publicKey);
        return claimAccount !== null ? claimAccount.claims : undefined;
    }
    getClaimProof(claimant, mint, quantity) {
        const quantityBuf = Buffer.alloc(8);
        quantityBuf.writeBigUInt64LE(quantity);
        const leaf = (0, keccak256_1.default)(Buffer.concat([claimant.toBuffer(), mint.toBuffer(), quantityBuf]));
        return this.merkleTree.getProof(leaf).map((p) => p.data);
    }
    async makeClaimInstructions(claimant, mint) {
        const ixs = [];
        const claims = await this.getClaims(claimant);
        const claimed = await this.getClaimed(claimant);
        const claim = claims.find((c) => c.mint.equals(mint));
        if (claim === undefined) {
            throw new Error('No claim for account and mint');
        }
        const destination = Token.getAssociatedTokenAddressSync(mint, claimant);
        ixs.push(Token.createAssociatedTokenAccountIdempotentInstruction(this.program.provider.publicKey, destination, this.program.provider.publicKey, mint));
        const proof = this.getClaimProof(claimant, mint, claim.quantity);
        ixs.push(await this.program.methods
            // @ts-ignore
            .claim(new anchor_1.BN(claim.quantity.toString()), proof)
            .accounts({
            distribution: this.publicKey,
            vault: this.findVaultAddress(mint),
            tokenAccount: destination,
            claimAccount: this.findClaimAccountAddress(claimant),
            mint: mint,
            claimant: claimant,
            tokenProgram: Token.TOKEN_PROGRAM_ID,
        })
            .instruction());
        return ixs;
    }
    async makeClaimTransaction(claimant, mint) {
        const transaction = new anchor_1.web3.Transaction().add(...(await this.makeClaimInstructions(claimant, mint)));
        return transaction;
    }
    async claim(mint) {
        return await this.program.provider.sendAndConfirm(await this.makeClaimTransaction(this.program.provider.publicKey, mint));
    }
    async transferToVault(mint, amount) {
        const source = Token.getAssociatedTokenAddressSync(mint, this.program.provider.publicKey);
        const destination = this.findVaultAddress(mint);
        return await this.program.provider.sendAndConfirm(new anchor_1.web3.Transaction().add(Token.createTransferInstruction(source, destination, this.program.provider.publicKey, amount)));
    }
    async closeVault(mint, destination) {
        const vault = this.findVaultAddress(mint);
        if (!destination) {
            destination = Token.getAssociatedTokenAddressSync(mint, this.program.provider.publicKey);
            try {
                await Token.getAccount(this.program.provider.connection, destination);
            }
            catch {
                this.program.provider.sendAndConfirm(new anchor_1.web3.Transaction().add(Token.createAssociatedTokenAccountIdempotentInstruction(this.program.provider.publicKey, destination, this.program.provider.publicKey, mint)));
            }
        }
        return await this.program.methods
            .vaultClose()
            .accounts({
            distribution: this.publicKey,
            vault,
            mint: mint,
            destination: destination,
            authority: this.program.provider.publicKey,
            systemProgram: anchor_1.web3.SystemProgram.programId,
            tokenProgram: Token.TOKEN_PROGRAM_ID,
        })
            .rpc();
    }
    async close() {
        const sigs = [];
        for (const mint of this.metadata.mints) {
            try {
                const sig = await this.closeVault(new anchor_1.web3.PublicKey(mint.address));
                sigs.push(sig);
            }
            catch (e) {
                console.error(e);
            }
        }
        sigs.push(await this.program.methods
            .distributionClose()
            .accounts({
            distribution: this.publicKey,
            solDestination: this.program.provider.publicKey,
            authority: this.program.provider.publicKey,
            systemProgram: anchor_1.web3.SystemProgram.programId,
        })
            .rpc());
        return sigs;
    }
    async closeClaimAccount() { }
}
exports.Distribution = Distribution;
