import { BN, Program, web3, } from '@coral-xyz/anchor';
import * as Token from '@solana/spl-token';
import { IDL, } from './mango_mints_redemption';
import { Distribution } from './accounts/distribution';
// TODO
// Edit distribution
// Close claims accounts for expired distributions
export const MANGO_MINTS_REDEMPTION_ID = new web3.PublicKey('3BUZXy9mPcsSCoxJQiBu2xxpMP6HEvFMZbaL5CAWwLUf');
export class MangoMintsRedemptionClient {
    provider;
    programId;
    program;
    constructor(provider, programId = MANGO_MINTS_REDEMPTION_ID) {
        this.provider = provider;
        this.programId = programId;
        this.program = new Program(IDL, programId, provider);
    }
    async loadDistribution(number) {
        const distributionNum = Buffer.alloc(4);
        distributionNum.writeUint32LE(number);
        const [publicKey] = await web3.PublicKey.findProgramAddress([Buffer.from('Distribution'), distributionNum], this.programId);
        const distribution = await this.loadDistributionFromPublicKey(publicKey);
        await distribution.loadMetadata();
        return distribution;
    }
    async loadDistributionFromPublicKey(publicKey) {
        const distribution = await this.program.account.distribution.fetch(publicKey);
        return new Distribution(this.programId, this.program, publicKey, distribution.distributionNum, distribution.merkleRoot, distribution.authority, distribution.start, distribution.duration, distribution.metadata);
    }
    async createDistribution(number, authority = this.provider.wallet.publicKey, start, duration, metadata, metadataUrl) {
        const distribution = Distribution.findAddress(number, this.programId);
        const merkleTree = metadata.toMerkleTree();
        const metadataUrlBuf = Buffer.alloc(256);
        metadataUrlBuf.write(metadataUrl);
        await this.program.methods
            .distributionCreate(number, [...merkleTree.getRoot()], new BN(start.getTime() / 1000), new BN(duration), [...metadataUrlBuf])
            .accounts({
            distribution,
            payer: this.provider.wallet.publicKey,
            authority,
            systemProgram: web3.SystemProgram.programId,
        })
            .rpc();
        for (const mint of metadata.mints) {
            console.log(mint);
            const mintPubkey = new web3.PublicKey(mint.address);
            const vault = web3.PublicKey.findProgramAddressSync([
                Buffer.from('Vault'),
                distribution.toBuffer(),
                mintPubkey.toBuffer(),
            ], this.programId)[0];
            await this.program.methods
                .vaultCreate()
                .accounts({
                distribution,
                vault,
                mint: mintPubkey,
                payer: this.provider.wallet.publicKey,
                tokenProgram: Token.TOKEN_PROGRAM_ID,
                associatedTokenProgram: Token.ASSOCIATED_TOKEN_PROGRAM_ID,
                systemProgram: web3.SystemProgram.programId,
                rent: web3.SYSVAR_RENT_PUBKEY,
            })
                .rpc();
        }
    }
    async editDistribution() { }
}
